︠c1475f7a-adb2-4b7b-8f0a-25cbdb0cb804i︠
%md
# Feb 5, 2016: An Algorithm to Compute the Ring of Integers (conclusion)
## William Stein

Today we will finish discussing details about how to explicitly turn the theoretical discussion from Monday and Wednesday into
an explicit algorithm to compute the ring $R$ of integers of $K$, noting that it gives an algorithm to compute a $p$-maximal
order, and that is often all we need for certain question.
︡d13491a9-d7d8-47e5-b152-754579a4c34d︡︡{"done":true,"md":"# Feb 5, 2016: An Algorithm to Compute the Ring of Integers (conclusion)\n## William Stein\n\nToday we will finish discussing details about how to explicitly turn the theoretical discussion from Monday and Wednesday into\nan explicit algorithm to compute the ring $R$ of integers of $K$, noting that it gives an algorithm to compute a $p$-maximal\norder, and that is often all we need for certain question."}
︠5fc51a8c-6686-4f6b-bec3-cc6c933019c7i︠
%md

## 1. Recall -- here's what we did on Monday and Wednesday:

Given $K = \QQ(\alpha)$, where $\alpha$ is a root of the irreducible integral monic $f$,
the ring $S=\ZZ[\alpha]$ is an order (=subring of full rank) in the ring $R$ of integers of $K$.

Given an order $S$ and a prime $p$, let
$$
   T = \{x \in K : xI_p \subset I_p\},
$$
where $I_p = \{ x \in S : x^m \in pS\text{ for some } m\geq 1\}$ is the $p$-radical of $S$.

Then we proved that $S \subset T \subset \frac{1}{p}S$ and $S=T$ if and only if $S$ is $p$-maximal.

We also proved that $T = \frac{1}{p}U$ where
$$
  U = \ker(S \to \text{End}(I_p / p I_p)).
$$

Our goal for today is to see how to compute $T$.  To really understand everything you need to know about Hermite normal form, and basically develop the analogue of everything you do in a linear algebra course, but over $\ZZ$ instead of a field.  We will not do all of that, due to lack of time.
︡e4d226f7-720b-41d4-b0e2-b2861fc7788e︡︡{"done":true,"md":"\n## 1. Recall -- here's what we did on Monday and Wednesday:\n\nGiven $K = \\QQ(\\alpha)$, where $\\alpha$ is a root of the irreducible integral monic $f$,\nthe ring $S=\\ZZ[\\alpha]$ is an order (=subring of full rank) in the ring $R$ of integers of $K$.\n\nGiven an order $S$ and a prime $p$, let\n$$\n   T = \\{x \\in K : xI_p \\subset I_p\\},\n$$\nwhere $I_p = \\{ x \\in S : x^m \\in pS\\text{ for some } m\\geq 1\\}$ is the $p$-radical of $S$.\n\nThen we proved that $S \\subset T \\subset \\frac{1}{p}S$ and $S=T$ if and only if $S$ is $p$-maximal.\n\nWe also proved that $T = \\frac{1}{p}U$ where\n$$\n  U = \\ker(S \\to \\text{End}(I_p / p I_p)).\n$$\n\nOur goal for today is to see how to compute $T$.  To really understand everything you need to know about Hermite normal form, and basically develop the analogue of everything you do in a linear algebra course, but over $\\ZZ$ instead of a field.  We will not do all of that, due to lack of time."}
︠eb8fea0b-f5a9-4c49-9749-158d56338a2fi︠
%md

## 2. Computing $I_p$.

**Lemma** (see Lemma 6.1.6 of Cohen GTM 138): If $p^j \geq [K:\QQ]$ then $I_p/ p I_p \subset S/pS$ is
the kernel of $x\mapsto x^{p^j}$.

Proof: exercise.

So to compute $I_p$, we take a $\ZZ$-basis $b_1,\ldots, b_n$ of $S$, raise each of these $b_i$ to the power $p^j$
and write down the matrix of whose coefficients $a_{i,j}$ are such that $b_i^{p^j} = \sum a_{i,j} b_i$.
Then reduce this matrix modulo $p$ and compute its kernel.

The result is a basis of $I_p /  p I_p$.

Then $I_p$ is got by taking the $\ZZ$-submodule of $K$ generated by any lift of this basis together with $pS$.

Let's do it!
︡710c85b0-65d6-40c3-bc49-7fcc8f45761c︡︡{"done":true,"md":"\n## 2. Computing $I_p$.\n\n**Lemma** (see Lemma 6.1.6 of Cohen GTM 138): If $p^j \\geq [K:\\QQ]$ then $I_p/ p I_p \\subset S/pS$ is\nthe kernel of $x\\mapsto x^{p^j}$.\n\nProof: exercise.\n\nSo to compute $I_p$, we take a $\\ZZ$-basis $b_1,\\ldots, b_n$ of $S$, raise each of these $b_i$ to the power $p^j$\nand write down the matrix of whose coefficients $a_{i,j}$ are such that $b_i^{p^j} = \\sum a_{i,j} b_i$.\nThen reduce this matrix modulo $p$ and compute its kernel.\n\nThe result is a basis of $I_p /  p I_p$.\n\nThen $I_p$ is got by taking the $\\ZZ$-submodule of $K$ generated by any lift of this basis together with $pS$.\n\nLet's do it!"}
︠a3ebcaf2-3ede-44eb-9947-bdb003ac905eas︠
%auto
def log(s, args):
    print s, args

def p_radical(S, p):
    """
    INPUT:
       - S -- an order in a number field K (assumed to be an absolute number field!)
       - p -- a prime number

    OUTPUT: the p-radical of S.
    """
    K = S.number_field()
    n = K.degree()
    log("n =", n)

    pj = p
    while pj < n:
        pj *= p
    log("p^j =", pj)

    powers = [x^pj for x in S.basis()]  # this could be optimized by reducing each time -- would be the bottlekneck right now
    log("powers =", powers)

    M = S.free_module()
    coords = [M.coordinates(list(y)) for y in powers]
    log("coords =", coords)

    A = matrix(GF(p), n, coords)
    log("matrix =", A)

    gens = []
    V = A.kernel()
    log("Kernel =", V)

    for b in V.basis():
        gens.append(M.linear_combination_of_basis(b.lift()))

    return M*p + M.span(gens)
︡d28d948c-7606-48aa-9ced-9d6b93f97824︡{"auto":true}︡
︠331cce6b-4fa3-49d9-b319-cc20ebc7633f︠
K.<a> = NumberField(x^3 - 4)
S = K.order([2*a])
S
S.basis()
︡72a81184-b16e-4359-a88a-f03fc2937851︡︡{"stdout":"Order in Number Field in a with defining polynomial x^3 - 4\n","done":false}︡{"stdout":"[1, 2*a, 4*a^2]\n","done":false}︡{"done":true}
︠aa004b95-91b1-48e6-8770-1fd1118a3d42︠
p = 2
I_p = p_radical(S, p); Ip
︡35e1d65e-8191-4fe4-869c-0808d7e75b72︡︡{"stdout":"n = 3\np^j = 4\npowers = [1, 64*a, 4096*a^2]\ncoords = [[1, 0, 0], [0, 32, 0], [0, 0, 1024]]\nmatrix = [1 0 0]\n[0 0 0]\n[0 0 0]\nKernel = Vector space of degree 3 and dimension 2 over Finite Field of size 2\nBasis matrix:\n[0 1 0]\n[0 0 1]\n","done":false}︡{"done":false,"stderr":"Error in lines 2-2\nTraceback (most recent call last):\n  File \"/projects/sage/sage-6.10/local/lib/python2.7/site-packages/smc_sagews/sage_server.py\", line 905, in execute\n    exec compile(block+'\\n', '', 'single') in namespace, locals\n  File \"\", line 1, in <module>\nNameError: name 'Ip' is not defined\n"}︡{"done":true}
︠59db4881-a070-48ce-8f25-cb698dea6a04i︠
%md
## 3. Compute the kernel $U$

Now that we have an explicit basis for $I_p$ as a $\ZZ$-submodule of $S$,
we move on to computing the kernel of $\phi: S \to \text{End}(I_p / p I_p)=M_{n\times n}(\FF_p)$ explicitly.

Since $pS$ goes to $0$, we instead compute the action of $S/pS$ on the $\FF_p$-vector subspace
$\text{End}(I_p / p I_p)$, then lift and combine with a basis for $pS$ to get $U$.  This
is just like what we did above to go from a kernel to having the full $I_p$.

First, we compute the linear map $\bar{\phi} : S/pS \to \text{End}(I_p / p I_p)$ which is from an $\FF_p$ vector
space of dimension $n$ to one of $n^2$, so represented by a matrix with $n^3$ entries.  (Cohen's book says "Unfortunately, in the round 2 algorithm, it seems unavoidable to use such large matrices.")

︡eeb7b82d-7b5b-4a55-9255-b829ed403d5f︡︡{"done":true,"md":"## 3. Compute the kernel $U$\n\nNow that we have an explicit basis for $I_p$ as a $\\ZZ$-submodule of $S$,\nwe move on to computing the kernel of $\\phi: S \\to \\text{End}(I_p / p I_p)=M_{n\\times n}(\\FF_p)$ explicitly.\n\nSince $pS$ goes to $0$, we instead compute the action of $S/pS$ on the $\\FF_p$-vector subspace\n$\\text{End}(I_p / p I_p)$, then lift and combine with a basis for $pS$ to get $U$.  This\nis just like what we did above to go from a kernel to having the full $I_p$.\n\nFirst, we compute the linear map $\\bar{\\phi} : S/pS \\to \\text{End}(I_p / p I_p)$ which is from an $\\FF_p$ vector\nspace of dimension $n$ to one of $n^2$, so represented by a matrix with $n^3$ entries.  (Cohen's book says \"Unfortunately, in the round 2 algorithm, it seems unavoidable to use such large matrices.\")"}
︠eb1c8a8d-4306-441e-96fc-0a624e6a77cfas︠
%auto
def phi_bar(S, p, I_p):
    # For each basis element of S, see how it acts on I_p modulo p*I_p.
    B = I_p.basis()  # basis of a free module
    log("I_p has basis ", B)
    B2 = [S(b) for b in B]
    log("Moving these elements into S for arithmetic: ", B2)
    rows = []
    for g in S.gens():
        row = sum([I_p.coordinates(list(g*c)) for c in B2], [])
        rows.append(row)
    log("rows of matrix: ", rows)
    phi_matrix = matrix(GF(p), rows)
    return phi_matrix

︡2e74968a-981d-440b-af9b-f780ed4a4c3b︡{"auto":true}︡
︠2af4c14f-605b-4c3b-bc75-07ff972de8a0︠
m = phi_bar(S, p, I_p)
m
︡cddf3e44-14c1-4030-8464-1be2592ac025︡︡{"stdout":"I_p has basis  [\n(2, 0, 0),\n(0, 2, 0),\n(0, 0, 4)\n]\nMoving these elements into S for arithmetic:  [2, 2*a, 4*a^2]\nrows of matrix:  [[1, 0, 0, 0, 1, 0, 0, 0, 1], [0, 2, 0, 0, 0, 1, 16, 0, 0], [0, 0, 2, 16, 0, 0, 0, 32, 0]]\n","done":false}︡{"stdout":"[1 0 0 0 1 0 0 0 1]\n[0 0 0 0 0 1 0 0 0]\n[0 0 0 0 0 0 0 0 0]\n","done":false}︡{"done":true}
︠cf014978-e828-481a-944c-78f5a41ba5ff︠
m.kernel()
︡7fb0e4f5-918d-4815-89ed-29c924d1b49e︡︡{"stdout":"Vector space of degree 3 and dimension 1 over Finite Field of size 2\nBasis matrix:\n[0 0 1]\n","done":false}︡{"done":true}
︠e5d58815-0164-416f-8746-4241ce7b8a3das︠
%auto
def compute_U(S, p, I_p):
    mat = phi_bar(S, p, I_p)
    log("the matrix is ", mat)

    V = mat.kernel()
    log("kernel ", V)

    M = S.free_module()
    gens = []
    for b in V.basis():
        gens.append(M.linear_combination_of_basis(b.lift()))
    log("lift of kernel elements", gens)

    U = M*p + M.span(gens)
    log("adding together with p*S yields", U)

    return U
︡82f30e30-46d2-41c9-ad3f-6ff134e710d9︡{"auto":true}︡
︠f5db8020-0cef-4e43-af38-1d4b65fadb67︠
U = compute_U(S, p, I_p)
︡af5fabc1-6879-412f-8fd6-5f3fb1dd15ec︡︡{"stdout":"I_p has basis  [\n(2, 0, 0),\n(0, 2, 0),\n(0, 0, 4)\n]\nMoving these elements into S for arithmetic:  [2, 2*a, 4*a^2]\nrows of matrix:  [[1, 0, 0, 0, 1, 0, 0, 0, 1], [0, 2, 0, 0, 0, 1, 16, 0, 0], [0, 0, 2, 16, 0, 0, 0, 32, 0]]\nthe matrix is  [1 0 0 0 1 0 0 0 1]\n[0 0 0 0 0 1 0 0 0]\n[0 0 0 0 0 0 0 0 0]\nkernel  Vector space of degree 3 and dimension 1 over Finite Field of size 2\nBasis matrix:\n[0 0 1]\nlift of kernel elements [(0, 0, 4)]\nadding together with p*S yields Free module of degree 3 and rank 3 over Integer Ring\nEchelon basis matrix:\n[2 0 0]\n[0 4 0]\n[0 0 4]\n","done":false}︡{"done":true}
︠44fa4786-115c-49d4-9528-d4993fcf0600as︠
%auto
def compute_T_from_U(S, U, p):
    return K.order([K(b/p) for b in U.basis()])
︡3242520e-a4ae-4541-a059-7e4ac1251434︡{"auto":true}︡
︠ceeab480-67f7-490e-965f-f3c27cdb07ee︠
T = compute_T_from(S, U, p)
T
T.basis()
︡2e18be96-c009-465c-863f-314abce92e94︡︡{"done":false,"stderr":"Error in lines 1-1\nTraceback (most recent call last):\n  File \"/projects/sage/sage-6.10/local/lib/python2.7/site-packages/smc_sagews/sage_server.py\", line 905, in execute\n    exec compile(block+'\\n', '', 'single') in namespace, locals\n  File \"\", line 1, in <module>\nNameError: name 'compute_T_from' is not defined\n"}︡{"done":true}
︠87efc04c-8bec-4cb2-9cba-6dd88f9dbd27︠
S.basis()
︡27d265e0-13a5-49e6-a6df-31b5518a06b6︡︡{"stdout":"[1, 2*a, 4*a^2]\n","done":false}︡{"done":true}
︠db146c9d-3687-4d46-9f19-19c07847c3f4︠
S.index_in(T)
︡84bb3a9a-0e03-478f-9986-0deca8398c90︡︡{"stdout":"2\n","done":false}︡{"done":true}
︠e37277d4-1da9-402f-bffa-fe7e200fc486i︠
%md
## 4. Put it all together.
︡38477de3-d8b9-4359-98bf-413eb5306ee3︡︡{"done":true,"md":"## 4. Put it all together."}
︠161ec4fb-4581-4baf-bff7-8fc594dc8866as︠
%auto
def compute_T(S, p):
    I_p = p_radical(S, p)
    U   = compute_U(S, p, I_p)
    T   = compute_T_from_U(S, U, p)
    return T
︡034b3f64-7a38-4bc2-a6e5-beb56a098cd7︡{"auto":true}︡
︠3d862d2a-82be-4e02-9b60-e0d9da1f8acc︠
K.<a> = NumberField(x^3 - 4)
S = K.order([2*a])
S2 = compute_T(S, 2)
︡4df562fd-6154-4c7a-8c39-98085ff2a94e︡︡{"stdout":"n = 3\np^j = 4\npowers = [1, 64*a, 4096*a^2]\ncoords = [[1, 0, 0], [0, 32, 0], [0, 0, 1024]]\nmatrix = [1 0 0]\n[0 0 0]\n[0 0 0]\nKernel = Vector space of degree 3 and dimension 2 over Finite Field of size 2\nBasis matrix:\n[0 1 0]\n[0 0 1]\nI_p has basis  [\n(2, 0, 0),\n(0, 2, 0),\n(0, 0, 4)\n]\nMoving these elements into S for arithmetic:  [2, 2*a, 4*a^2]\nrows of matrix:  [[1, 0, 0, 0, 1, 0, 0, 0, 1], [0, 2, 0, 0, 0, 1, 16, 0, 0], [0, 0, 2, 16, 0, 0, 0, 32, 0]]\nthe matrix is  [1 0 0 0 1 0 0 0 1]\n[0 0 0 0 0 1 0 0 0]\n[0 0 0 0 0 0 0 0 0]\nkernel  Vector space of degree 3 and dimension 1 over Finite Field of size 2\nBasis matrix:\n[0 0 1]\nlift of kernel elements [(0, 0, 4)]\nadding together with p*S yields Free module of degree 3 and rank 3 over Integer Ring\nEchelon basis matrix:\n[2 0 0]\n[0 4 0]\n[0 0 4]\n","done":false}︡{"done":true}
︠af8d8736-e45b-4d69-86d1-5670e487b3c8︠
S.index_in(S2)
︡735964ac-aa9d-436d-8878-3512936564c7︡︡{"stdout":"2\n","done":false}︡{"done":true}
︠0d185344-89a4-43d3-885a-fae9e6e822e0︠
S3 = compute_T(S2, 2)
S3.basis()
︡d6a70d34-838b-4f10-abf8-57c3c4f4df1b︡︡{"stdout":"n = 3\np^j = 4\npowers = [1, 64*a, 256*a^2]\ncoords = [[1, 0, 0], [0, 32, 0], [0, 0, 128]]\nmatrix = [1 0 0]\n[0 0 0]\n[0 0 0]\nKernel = Vector space of degree 3 and dimension 2 over Finite Field of size 2\nBasis matrix:\n[0 1 0]\n[0 0 1]\nI_p has basis  [\n(2, 0, 0),\n(0, 2, 0),\n(0, 0, 2)\n]\nMoving these elements into S for arithmetic:  [2, 2*a, 2*a^2]\nrows of matrix:  [[1, 0, 0, 0, 1, 0, 0, 0, 1], [0, 2, 0, 0, 0, 2, 8, 0, 0], [0, 0, 2, 8, 0, 0, 0, 8, 0]]\nthe matrix is  [1 0 0 0 1 0 0 0 1]\n[0 0 0 0 0 0 0 0 0]\n[0 0 0 0 0 0 0 0 0]\nkernel  Vector space of degree 3 and dimension 2 over Finite Field of size 2\nBasis matrix:\n[0 1 0]\n[0 0 1]\nlift of kernel elements [(0, 2, 0), (0, 0, 2)]\nadding together with p*S yields Free module of degree 3 and rank 3 over Integer Ring\nEchelon basis matrix:\n[2 0 0]\n[0 2 0]\n[0 0 2]\n","done":false}︡{"stdout":"[1, a, a^2]\n","done":false}︡{"done":true}
︠1b405d8d-b592-4b40-95da-3e16f5e1141c︠
S2.index_in(S3)
︡95c740f4-3bc3-4ed4-81e3-798f7049c29e︡︡{"stdout":"4\n","done":false}︡{"done":true}
︠d5d1f823-c6ee-471f-b7b3-7973eed6a7f3︠
S.index_in(S3)
︡80361bf3-16ac-41f4-a8e8-7ee39ead3aa9︡︡{"stdout":"8\n","done":false}︡{"done":true}
︠e60c04a0-068a-4bf0-af48-56eb9712c33b︠
S3.free_module() / S.free_module()
︡25132e24-1a08-46c4-ab59-7970da6f8f7e︡︡{"stdout":"Finitely generated module V/W over Integer Ring with invariants (2, 4)\n","done":false}︡{"done":true}
︠35bae064-fdd8-418a-907e-2c2d0d65bd5c︠
S4 = compute_T(S3, 2)
S4
︡af17520d-76f0-4241-b8c5-637f85d123e8︡︡{"stdout":"n = 3\np^j = 4\npowers = [1, 4*a, 16*a^2]\ncoords = [[1, 0, 0], [0, 4, 0], [0, 0, 16]]\nmatrix = [1 0 0]\n[0 0 0]\n[0 0 0]\nKernel = Vector space of degree 3 and dimension 2 over Finite Field of size 2\nBasis matrix:\n[0 1 0]\n[0 0 1]\nI_p has basis  [\n(2, 0, 0),\n(0, 1, 0),\n(0, 0, 1)\n]\nMoving these elements into S for arithmetic:  [2, a, a^2]\nrows of matrix:  [[1, 0, 0, 0, 1, 0, 0, 0, 1], [0, 2, 0, 0, 0, 1, 2, 0, 0], [0, 0, 2, 2, 0, 0, 0, 4, 0]]\nthe matrix is  [1 0 0 0 1 0 0 0 1]\n[0 0 0 0 0 1 0 0 0]\n[0 0 0 0 0 0 0 0 0]\nkernel  Vector space of degree 3 and dimension 1 over Finite Field of size 2\nBasis matrix:\n[0 0 1]\nlift of kernel elements [(0, 0, 1)]\nadding together with p*S yields Free module of degree 3 and rank 3 over Integer Ring\nEchelon basis matrix:\n[2 0 0]\n[0 2 0]\n[0 0 1]\n","done":false}︡{"stdout":"Order in Number Field in a with defining polynomial x^3 - 4\n","done":false}︡{"done":true}
︠64a9dad4-8952-4364-b9d7-5cd10191c6ce︠
︡617f2d4c-2c8d-474d-b932-b4f5e57d6c85︡︡{"done":true}
︠ec10cd07-8362-442a-b2cc-e2c6727ab913︠
S3.index_in(S4)
︡a9ae6ee1-512a-474f-b65b-03e4c0b28a45︡︡{"stdout":"2\n","done":false}︡{"done":true}
︠9bb0a8b5-917b-4ac3-99f3-c96023dd0ada︠

S5 = compute_T(S4, 2)
︡fa39237b-7d68-4aa8-80b9-c47d28924f90︡︡{"stdout":"n = 3\np^j = 4\npowers = [1, 4*a, a^2]\ncoords = [[1, 0, 0], [0, 4, 0], [0, 0, 2]]\nmatrix = [1 0 0]\n[0 0 0]\n[0 0 0]\nKernel = Vector space of degree 3 and dimension 2 over Finite Field of size 2\nBasis matrix:\n[0 1 0]\n[0 0 1]\nI_p has basis  [\n(2, 0, 0),\n(0, 1, 0),\n(0, 0, 1/2)\n]\nMoving these elements into S for arithmetic:  [2, a, 1/2*a^2]\nrows of matrix:  [[1, 0, 0, 0, 1, 0, 0, 0, 1], [0, 2, 0, 0, 0, 2, 1, 0, 0], [0, 0, 2, 1, 0, 0, 0, 1, 0]]\nthe matrix is  [1 0 0 0 1 0 0 0 1]\n[0 0 0 0 0 0 1 0 0]\n[0 0 0 1 0 0 0 1 0]\nkernel  Vector space of degree 3 and dimension 0 over Finite Field of size 2\nBasis matrix:\n[]\nlift of kernel elements []\nadding together with p*S yields Free module of degree 3 and rank 3 over Integer Ring\nEchelon basis matrix:\n[2 0 0]\n[0 2 0]\n[0 0 1]\n","done":false}︡{"done":true}
︠b828c711-6bc8-480a-8ea7-8d6eb83d962b︠
S5.index_in(S4)
︡c504153b-469c-4001-bef0-5f74dbbb24a2︡︡{"stdout":"1\n","done":false}︡{"done":true}
︠fd77a104-5676-4893-a6e0-0b056c2cc935︠
S
︡8fdd77b0-322d-4875-99c2-70b38d929a96︡︡{"stdout":"Order in Number Field in a with defining polynomial x^3 - 4\n","done":false}︡{"done":true}
︠3d0314d5-7cfc-475f-a0bc-df2936092770︠
S2
︡0761f4e3-e68f-4799-9135-288ad8421758︡︡{"stdout":"Order in Number Field in a with defining polynomial x^3 - 4\n","done":false}︡{"done":true}
︠b8a0ea73-aeea-47c1-b5d8-92d6135d7f40︠
S3
︡480f33ab-f720-4daa-9a81-aa4fee5ddbc3︡︡{"stdout":"Order in Number Field in a with defining polynomial x^3 - 4\n","done":false}︡{"done":true}
︠77b0fee9-8426-4070-8a4f-e64c01847fd6︠
S4
︡a5201cc5-269c-419d-b656-3e7b6fddd058︡︡{"stdout":"Order in Number Field in a with defining polynomial x^3 - 4\n","done":false}︡{"done":true}
︠4eb8babe-edf3-487a-b9bf-db87722392b5︠
S5
︡5e8d1f88-6fe7-4fea-af23-135df96cf4ec︡︡{"stdout":"Order in Number Field in a with defining polynomial x^3 - 4\n","done":false}︡{"done":true}
︠6f726b14-f404-4f30-a9e2-c5fb13832722︠









